<?php
/*
diagnose = array (
	'caution', 'warning', 'untested', 'safe'
);
*/
class Diagnostic extends Request
{
    private static $mcUrl = 'http://www.siteadvisor.com/sites/%s';
    private static $glUrl = 'https://www.google.com/safebrowsing/diagnostic?output=jsonp&site=%s';
    private static $norUrl = 'http://safeweb.norton.com/report/show?url=%s';
    private static $avgUrl = 'http://www.avgthreatlabs.com/us-en/website-safety-reports/domain/%s';

    public static function google($domain)
    {
        $url = sprintf(self::$glUrl, $domain);
        $curl = curl_init($url);
        $headers = array(
            'X-Requested-With' => 'XMLHttpRequest',
        );
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 20);
        curl_setopt($curl, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.2) Gecko/20090729 Firefox/3.5.2 GTB5');
        $output = parent::curl_exec($curl);
        $response = !curl_errno($curl) ? $output : false;
        curl_close($curl);

        if(!$response) {
            return 'untested';
        }
        if(!$jsonp = substr($response, strpos($response, '('))) {
            return 'untested';
        }
        if(!$data = @json_decode(trim($jsonp,'();'), true)) {
            return 'untested';
        }
        if(!isset($data['website'], $data['numTested'])) {
            return 'untested';
        }
        $website = $data['website'];
        if(!isset($website['malwareListStatus'], $website['uwsListStatus'], $website['socialListStatus'])) {
            return 'untested';
        }

        $toValidate = array($website['malwareListStatus'], $website['uwsListStatus'], $website['socialListStatus']);
        if(in_array("listed", $toValidate)) {
            return "caution";
        }
        if(in_array("partial", $toValidate)) {
            return "warning";
        }
        if($data['numTested']===0) {
            return 'untested';
        }
        return "safe";
    }

    public static function mcafee($domain)
    {
        static $inc = 0;
        $max = 3;
        $inc++;
        $state = array(
            'siteRed'=>'caution',
            'siteGray'=>'untested',
            'siteGreen'=>'safe',
        );
        $url = sprintf(self::$mcUrl, $domain);
        if(!$response = parent::run($url)) {
            return $inc >= $max ? 'untested' : self::mcafee($domain);
        }
        preg_match('#<div(.*?)id="siteVerdict"(.*?)class="(.*?)"(.*?)>#is', $response, $matches);
        $diagnose = isset($matches[3]) ? trim($matches[3]) : 'untested';
        if(!isset($state[$diagnose])) {
            return $inc >= $max ? 'untested' : self::mcafee($domain);
        } else {
            return $state[$diagnose];
        }
        return isset($state[$diagnose]) ? $state[$diagnose] : 'untested';
    }

    public static function avg($domain) {
        $url = sprintf(self::$avgUrl, $domain);
        $diagnose = array(
            'green' => 'safe',
            'yellow' => 'warning',
            'orange' => 'warning',
            'red' => 'caution',
            'gray' => 'untested',
        );
        if(!$response = parent::run($url))
            return 'untested';

        preg_match('#<span id="linkscanner_icon" class="linkscanner (.+?)" (.+?)></span>#is', $response, $matches);
        $d = isset($matches[1]) ? trim($matches[1]) : 'untested';
        return isset($diagnose[$d]) ? $diagnose[$d] : 'untested';
    }

    public static function norton($domain) {
        $url = sprintf(self::$norUrl, $domain);
        $diagnose = array(
            'icoSafe' => 'safe',
            'icoUntested' => 'untested',
            'icoWarning' => 'warning',
            'icoCaution' => 'caution',
            'icoNSecured' => 'safe',
        );
        if(!$response = parent::run($url))
            return 'untested';
        preg_match('#<img(?:[^>]*)class="big_clip (.*?)"(?:[^>]*)>#is', $response, $matches);
        $d = isset($matches[1]) ? trim($matches[1]) : 'untested';
        return isset($diagnose[$d]) ? $diagnose[$d] : 'untested';
    }
}